@@ -18,6 +18,10 @@
 #include "mt76.h"
 #include "../bus/bus.h"
 
+ushort fw_override_pid = 0;
+MODULE_PARM_DESC(fw_override, "Use firmware for the provided product ID instead of the one detected automatically");
+module_param_named(fw_override, fw_override_pid, ushort, 0600);
+
 #define XONE_DONGLE_NUM_IN_URBS 12
 #define XONE_DONGLE_NUM_OUT_URBS 12
 
@@ -304,16 +308,10 @@ static ssize_t xone_dongle_pairing_store(struct device *dev,
 	if (err)
 		return err;
 
-	err = pm_runtime_resume_and_get(dev);
-	if (err)
-		return err;
-
 	err = xone_dongle_toggle_pairing(dongle, enable);
 	if (err)
 		return err;
 
-	pm_runtime_put(dev);
-
 	return count;
 }
 
@@ -912,8 +910,14 @@ static void xone_dongle_fw_load(struct work_struct *work)
 	const struct firmware *fw;
 	char fwname[21];
 	int err;
+	u16 fw_product = dongle->product;
 
-	sprintf(fwname, "xone_dongle_%04x.bin", dongle->product);
+	if (fw_override_pid) {
+		dev_info(mt->dev, "Firmware overriden with PID=0x%04x", fw_override_pid);
+		fw_product = fw_override_pid;
+	}
+
+	sprintf(fwname, "xone_dongle_%04x.bin", fw_product);
 	err = xone_dongle_fw_requester(&fw, dongle, fwname);
 	if (dongle->fw_state == XONE_DONGLE_FW_STATE_STOP_LOADING) {
 		dongle->fw_state = XONE_DONGLE_FW_STATE_ERROR;
@@ -929,7 +933,12 @@ static void xone_dongle_fw_load(struct work_struct *work)
 	dev_dbg(mt->dev, "%s: firmware requested successfully\n", __func__);
 
 
-	err = xone_mt76_load_firmware(mt, fw);
+	for (int i = 0; i < 5; ++i) {
+		err = xone_mt76_load_firmware(mt, fw);
+		if (!err)
+			break;
+		ssleep(1);
+	}
 	release_firmware(fw);
 	if (err) {
 		dongle->fw_state = XONE_DONGLE_FW_STATE_ERROR;
@@ -1159,6 +1168,8 @@ static int xone_dongle_resume(struct usb_interface *intf)
 		return 0;
 	}
 
+	msleep(1500);
+
 	while ((urb = usb_get_from_anchor(&dongle->urbs_in_idle))) {
 		usb_anchor_urb(urb, &dongle->urbs_in_busy);
 		usb_free_urb(urb);
@@ -1167,6 +1178,10 @@ static int xone_dongle_resume(struct usb_interface *intf)
 		if (err)
 			return err;
 	}
+
+	msleep(1000);
+	return xone_mt76_resume_radio(&dongle->mt);
+	msleep(500);
 	return xone_mt76_resume_radio(&dongle->mt);
 }
 
@@ -1292,5 +1307,5 @@ module_usb_driver(xone_dongle_driver);
 MODULE_DEVICE_TABLE(usb, xone_dongle_id_table);
 MODULE_AUTHOR("Severin von Wnuck-Lipinski <severinvonw@outlook.de>");
 MODULE_DESCRIPTION("xone dongle driver");
-MODULE_VERSION("0.5.0");
+MODULE_VERSION("0.5.4");
 MODULE_LICENSE("GPL");
